#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""Description of the class in one sentence.

Description more in details.
"""
# Generic/Built-in modules
import logging
import os
import time
import datetime

# Third-party modules

# Owned modules


class SingletonType(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(SingletonType,
                                        cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class Log(object, metaclass=SingletonType):
    _logger = None
    _file_handler = None
    _stream_handler = None
    _formatter = None

    def __init__(self):
        self._logger = logging.getLogger("Logger")
        self._logger.setLevel(logging.INFO)
        #self._formatter = logging.Formatter(
        #    "%(asctime)-15s [%(levelname)8s] %(message)-100s (%(filename)s:%(lineno)s)",
        #    "%Y-%m-%d %H:%M:%S")
        self._formatter = logging.Formatter(
            "%(asctime)-8s [%(levelname)8s] %(message)s", "%H:%M:%S")

    def __del__(self):
        if self._file_handler is not None:
            self._file_handler.close()

    def set_file(self, cwd):
        if self._file_handler is not None:
            return

        self._file_handler = logging.FileHandler(
            os.path.join(cwd, "oftools_compile.log"))
        self._file_handler.setFormatter(self._formatter)
        self._logger.addHandler(self._file_handler)

    def set_level(self, level):

        if level == "DEBUG":
            self._formatter = logging.Formatter(
                "%(asctime)-8s [%(levelname)8s] %(message)s (%(filename)s:%(lineno)s)",
                "%H:%M:%S")
            self._logger.setLevel(logging.DEBUG)
        elif level == "INFO":
            self._logger.setLevel(logging.INFO)
        elif level == "WARNING":
            self._logger.setLevel(logging.WARNING)
        elif level == "ERROR":
            self._logger.setLevel(logging.ERROR)
        elif level == "CRITICAL":
            self._logger.setLevel(logging.CRITICAL)

        if self._stream_handler is None:
            self._stream_handler = logging.StreamHandler()
            self._stream_handler.setFormatter(self._formatter)
            self._logger.addHandler(self._stream_handler)

    def get(self):
        return self._logger

    def clear(self):
        # oftools_compile.log
        if self._file_handler is not None:
            self._logger.removeHandler(self._file_handler)
            self._file_handler.close()
            self._file_handler = None



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""Description of the class in one sentence.

Description more in details.
"""
# Generic/Built-in modules
import os
import shutil
import time
from datetime import datetime

# Third-party modules

# Owned modules
from .Job import Job
from .Log import Log
from .Context import Context


class SetupJob(Job):

    def _analyze(self):

        # check if workdir is defined in the profile
        if self._profile.has_option(self._section, 'workdir') is False:
            Log().get().critical('[' + self._section +
                                 '] cannot find workdir section in the profile')
            exit(-1)

        # check if workdir is accessable
        workdir = self._profile.get(self._section, 'workdir')
        workdir = os.path.expandvars(workdir)
        if os.path.isdir(workdir) is False:
            if os.access(workdir, os.W_OK) is False:
                Log().get().critical('[' + self._section +
                                     '] no write access on workdir = ' +
                                     workdir)
                exit(-1)

        # check if given section is already completed
        if Context().is_section_complete(self._section):
            return -1

        return 0

    def _process_workdir(self, workdir, in_name):

        # Create the name for the workdir by adding suffix to in_name
        workdir = os.path.expandvars(workdir)
        Context().set_root_workdir(workdir)

        try:
            file_name = in_name.rsplit('/', 1)[1]
        except:
            file_name = in_name

        cur_workdir = os.path.join(
            workdir,
            file_name + Context().get_const_tag() + Context().get_time_stamp())

        # create_workdir
        while True:
            if not os.path.isdir(cur_workdir):
                os.mkdir(cur_workdir)
                break

            Log().get().warning(
                cur_workdir +
                ' already exists. sleep 1 second to assign a new time stamp')
            time.sleep(1)
            Context().set_time_stamp()
            cur_workdir = os.path.join(
                workdir, file_name + Context().get_const_tag() +
                Context().get_time_stamp())

        # create_reportdir
        report_workdir = os.path.join(workdir, 'report')
        if not os.path.isdir(report_workdir):
            os.mkdir(report_workdir)

        # copy source to workdir
        shutil.copy(in_name, cur_workdir)

        # change directory to workdir
        os.chdir(cur_workdir)
        Context().set_cur_workdir(cur_workdir)

        # set log file handle
        Log().set_file(cur_workdir)

        header = '============================================================'
        header = header[:1] + ' ' + file_name + ' ' + header[len(file_name) +
                                                             2:]

        Log().get().info(header)
        Log().get().info('[' + self._section + '] ' + 'mkdir ' + cur_workdir)
        Log().get().info('[' + self._section + '] ' + 'cp ' + in_name + ' ' +
                         cur_workdir)
        Log().get().info('[' + self._section + '] ' + 'cd ' + cur_workdir)

        return file_name

    def run(self, in_name):
        # analyze section
        if self._analyze() < 0:
            Log().get().debug("[" + self._section + "] skip section")
            return in_name

        # update predefined environment variable
        try:
            out_name = in_name.rsplit('/', 1)[1]
        except:
            out_name = in_name
        base_name = self._remove_extension_name(out_name)
        Context().add_env('$OF_COMPILE_IN', out_name)
        Context().add_env('$OF_COMPILE_OUT', out_name)
        Context().add_env('$OF_COMPILE_BASE', base_name)

        # add environment variables and filters
        Log().get().debug("[" + self._section + "] process options")
        for key in self._profile.options(self._section):
            value = self._profile.get(self._section, key)

            if key.startswith('$'):
                Context().add_env(key, value)

            elif key.startswith('?'):
                self._add_filter(key, value, in_name)

            elif key == "workdir":
                out_name = self._process_workdir(value, in_name)

        # set the mandatory section
        sections = self._profile.sections()
        if "deploy" in sections:
            for section in reversed(sections):
                if section.startswith('deploy') is False:
                    Log().get().debug('[' + self._section +
                                      '] mandatory section: ' + section)
                    Context().set_mandatory_section(section)
                    break

        # set section as completed
        Context().set_section_complete(self._remove_filter_name(self._section))

        return out_name



#!/usr/bin/python3
# -*- coding: utf-8 -*-
import os
import re
import sys

from oftools_compile.Main import Main


#args = ["-p", "tests/test_listing/PROFILE1.txt", "-s", "tests/test_listing/add01.cob"]
def test_listing():
    pwd = os.getcwd() + '/tests/test_ofcob'
    sys.argv = [sys.argv[0]]
    sys.argv.append('-p')
    sys.argv.append(pwd + '/ok.prof')
    sys.argv.append('-s')
    sys.argv.append(pwd + '/add01.cob')

    return Main().run()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""Description of the class in one sentence.

Description more in details.
"""
# Generic/Built-in modules
import os
import argparse
import traceback
import sys
import time
from configparser import ConfigParser
from collections import OrderedDict

# Third-party modules

# Owned modules
from . import __version__
from .JobFactory import JobFactory
from .CompileJob import CompileJob
from .SetupJob import SetupJob
from .DeployJob import DeployJob
from .Log import Log
from .Context import Context
from .ReportGenerator import ReportGenerator


def main():
    return Main().run()


class Main:

    def __init__(self):
        return

    def _create_jobs(self, profile):
        jobs = []
        job_factory = JobFactory(profile)
        for section in profile.sections():
            try:
                job = job_factory.create(section)
                jobs.append(job)
            except:
                traceback.print_exc()
                print('Unexpected error detected during the job creation')
                exit(-1)

        # analyze created jobs
        if job_factory.is_fine() is not True:
            print('Missing jobs found. abort!')
            exit(-1)

        return jobs

    def _parse_arg(self):
        # add parse arguments
        arg_parser = argparse.ArgumentParser()

        arg_parser.add_argument(
            '-p',
            '--profile',
            action='append',
            help='profile which contains description of the compilation target.',
            required=False)

        arg_parser.add_argument('-s',
                                '--source',
                                action='append',
                                help='name of the source which must be a file.',
                                required=False)

        arg_parser.add_argument('-v',
                                '--version',
                                action='store_true',
                                help='print version information.',
                                required=False)

        arg_parser.add_argument(
            '-t',
            '--tag',
            help='add tag to the name of report file and the listing directory.',
            required=False)

        arg_parser.add_argument(
            '-l',
            '--log',
            help='set log level (DEBUG|INFO|WARNING|ERROR|CRITICAL).',
            required=False)

        # deprecated args
        arg_parser.add_argument('-r',
                                '--recursive',
                                action='store_true',
                                help=argparse.SUPPRESS)
        arg_parser.add_argument('-e',
                                '--export',
                                action='store_true',
                                help=argparse.SUPPRESS)

        # do the parsing
        args = arg_parser.parse_args()

        # analyze parsing result
        if args.version is True:
            return args

        if args.profile is None:
            Log().get().critical('-p or --profile option is not specified')
            exit(-1)

        if args.source is None:
            Log().get().critical('-s or --source option is not specified')
            exit(-1)

        if len(args.profile) != len(args.source):
            Log().get().critical(
                'the number of profile and source pairs does not match. profile='
                + str(len(args.profile)) + ',source=' + str(len(args.source)))
            exit(-1)

        for profile in args.profile:
            if os.path.isfile(os.path.expandvars(profile)) is False:
                Log().get().critical('cannot access profile: ' + profile)
                exit(-1)

        for source in args.source:
            if os.path.exists(os.path.expandvars(source)) is False:
                Log().get().critical('cannot access source: ' + source)
                exit(-1)

        return args

    def _run_internal(self, profile, source, report_generator):

        source_list = []

        # read profile
        profile_parser = ConfigParser(dict_type=OrderedDict)
        profile_parser.optionxform = str
        profile_parser.read(os.path.expandvars(profile))

        Log().get().debug('profile path = ' + os.path.expandvars(profile))

        # build source list
        if os.path.isdir(source):
            directory = os.path.expandvars(source)

            for dirpath, _, filenames in os.walk(directory):
                if dirpath.startswith('.'):
                    continue
                for f in filenames:
                    if f.startswith('.'):
                        continue
                    source_list.append(os.path.abspath(os.path.join(dirpath,
                                                                    f)))
        else:
            source_list = [source]

        source_list.sort()

        last_job = None
        for source in source_list:
            start_time = time.time()
            Context().clear()
            Log().clear()

            in_name = ""
            out_name = source
            jobs = self._create_jobs(profile_parser)

            for job in jobs:
                last_job = job
                try:
                    in_name = out_name
                    out_name = job.run(in_name)
                    rc = 0
                except KeyboardInterrupt:
                    rc = -255
                    break
                except:
                    trace_list = traceback.format_exc().splitlines()
                    for trace in trace_list:
                        Log().get().error(trace)
                    rc = -3
                    break

            # stop looping when ctrl+c
            if rc == -255:
                break

            unit_time = time.time() - start_time

            success = 'Y'
            if rc < 0:
                success = 'N'

            last_section = last_job._remove_filter_name(last_job.get_section())
            if last_section.startswith('deploy'):
                if Context().is_mandatory_complete() is not True:
                    last_section = Context().get_mandatory_section()

            report_generator.add(source,
                                 Context().get_cur_workdir(), last_section,
                                 success, unit_time)

        return rc

    def run(self):
        # initialize
        rc = 0

        # parse inline command
        args = self._parse_arg()

        if args.version is True:
            version = 'oftools-compile ' + __version__
            print(version)
            return 0

        # set log level
        Log().set_level(args.log)

        # run jobs through sources
        Context().set_const_tag(args.tag)
        report_generator = ReportGenerator()

        for i in range(len(args.source)):
            Log().clear()
            Context().clear()
            rc = self._run_internal(args.profile[i], args.source[i],
                                    report_generator)
            if rc is not 0:
                break

        report_generator.generate()

        # need to clear context to run pytest
        Log().clear()
        Context().clear()

        return rc



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""Description of the class in one sentence.

Description more in details.
"""
# Generic/Built-in modules

# Third-party modules

# Owned modules
from .DeployJob import DeployJob
from .CompileJob import CompileJob
from .SetupJob import SetupJob


class JobFactory:
    _is_setup = False
    _is_compile = False
    _is_deploy = False

    def __init__(self, profile):
        self._profile = profile
        return

    def create(self, section):
        if section is None:
            return None
        elif section == "setup":
            self._is_setup = True
            return SetupJob(section, self._profile)
        elif section.startswith("deploy"):
            self._is_deploy = True
            return DeployJob(section, self._profile)
        else:
            self._is_compile = True
            return CompileJob(section, self._profile)

    def is_fine(self):
        if self._is_setup is False:
            print('setup section missing in profile')
        if self._is_compile is False:
            print('compile section missing in profile')

        return self._is_setup and self._is_compile


#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""Description of the class in one sentence.

Description more in details.
"""
# Generic/Built-in modules
import os
import subprocess

# Third-party modules

# Owned modules
from .Log import Log
from .Context import Context


class Job(object):
    _section = None
    _profile = None

    def __init__(self, section, profile):
        self._section = section
        self._profile = profile

    def _add_filter(self, key, value, in_name):
        #Log().get().debug('add_filter: [' + key + ',' +
        #                  os.path.expandvars(value) + ']')

        result = False
        shell_cmd = os.path.expandvars(value)

        env = Context().get_env()
        proc = subprocess.Popen([shell_cmd],
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE,
                                shell=True,
                                env=env)
        out, err = proc.communicate()

        if b'' != out:
            Log().get().debug(err.decode('utf-8'))
        if b'' != err:
            Log().get().debug(out.decode('utf-8'))

        # grep returns 0 if line matches
        if proc.returncode == 0:
            result = True

        Context().add_filter_result(key, result)

    def _evaluate_filter(self, section, in_name):
        if "?" not in section:
            return True

        result = Context().get_filter_result(section)
        return result

    def _remove_extension_name(self, in_name):
        return in_name.rsplit('.', 1)[0]

    def _remove_filter_name(self, section):

        index = section.find('?')
        if index > 0:
            section = section[:index]

        return section

    def _resolve_filter_name(self, section):

        index = section.find('?')
        if index > 0:
            section = section[index:]

        return section

    def get_section(self):
        return self._section


#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""Description of the class in one sentence.

Description more in details.
"""
# Generic/Built-in modules
import os
import subprocess
import logging

# Third-party modules

# Owned modules
from .Log import Log
from .Context import Context


class Record(object):
    _rc = 0
    _source = ""
    _section = ""
    _unit_time = 0

    def __init__(self, source, list_dir, section, success, unit_time):
        self._source = source
        self._section = section
        self._success = success
        self._list_dir = list_dir
        self._unit_time = unit_time
        pass

    def to_csv(self):
        return str(self._source + ',' + self._list_dir + ',' + self._section +
                   ',' + self._success + ',' + str(round(self._unit_time, 4)))


class ReportGenerator(object):

    _total_time = 0
    _success_count = 0
    _fail_count = 0
    _records = []

    def __init__(self):
        return

    def add(self, source, list_dir, section, success, time):

        record = Record(source, list_dir, section, success, time)
        self._records.append(record)

        if success == 'Y':
            self._success_count += 1
        else:
            self._fail_count += 1

        self._total_time += time

        if success == 'Y':
            Log().get().info('BUILD SUCCESS (' + str(round(time, 4)) + ' sec)')
        else:
            Log().get().info('BUILD FAILED (' + str(round(time, 4)) + ' sec)')
        Log().get().info('')

        return

    def generate(self):

        results = []
        results.append("source,list_dir,section,success,time")

        Log().get().info(
            '= SUMMARY ==================================================')
        Log().get().info('TOTAL TIME: ' + str(round(self._total_time, 4)) +
                         ' sec')
        Log().get().info('TOTAL     : ' +
                         str(self._success_count + self._fail_count))
        Log().get().info('SUCCESS   : ' + str(self._success_count))
        Log().get().info('FAILED    : ' + str(self._fail_count))

        for record in self._records:
            results.append(record.to_csv())

        # export the report file
        file_name = 'report/oftools_compile' + Context().get_const_tag(
        ) + Context().get_time_stamp() + '.csv'
        file_name = os.path.expandvars(
            os.path.join(Context().get_root_workdir(), file_name))

        with open(file_name, 'w') as f:
            for result in results:
                f.write("%s\n" % result)
        f.close()

        Log().get().info('csv report has been generated: ' + file_name)

        return


#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""Description of the class in one sentence.

Description more in details.
"""
# Generic/Built-in modules
import shutil
import os
import subprocess

# Third-party modules

# Owned modules
from .Job import Job
from .Log import Log
from .Context import Context


class DeployJob(Job):

    def _analyze(self, in_name):

        # check if any compile job was a success
        if Context().is_mandatory_complete() is False:
            Log().get().error(
                'mandatory section [' + Context().get_mandatory_section() +
                '] did not ran successfully. aborting the deploy job')
            exit(-1)

        # check if given section is already completed
        if Context().is_section_complete(self._section):
            Log().get().debug('section has already been processed. skipping [' +
                              self._section + '] section.')
            return -1

        # evaluate filter to decide whether this section should run or not
        if self._evaluate_filter(self._section, in_name) is False:
            Log().get().debug('[' + self._section + '] ' +
                              self._resolve_filter_name(self._section) +
                              ' is False. skipping section.')
            return -1

        return 0

    def _process_region(self, out_name):

        if self._profile.has_option(self._section, 'region') is False:
            return

        regions = self._profile.get(self._section, 'region').split(':')
        for region in regions:
            shell_cmd = 'cp ' + out_name + ' ' + os.path.join(
                '$OPENFRAME_HOME/osc/region',
                os.path.expandvars(region) + '/tdl/mod')
            shell_cmd += '; '
            shell_cmd += 'osctdlupdate'
            shell_cmd += ' ' + region
            shell_cmd += ' ' + self._remove_extension_name(out_name)

            Log().get().info('[' + self._section + '] ' + shell_cmd)
            proc = subprocess.Popen([shell_cmd],
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE,
                                    shell=True)
            out, err = proc.communicate()

            # handle resultget
            if proc.returncode != 0:
                Log().get().error(err.decode('utf-8'))
                Log().get().error(out.decode('utf-8'))
                exit(proc.returncode)

        return

    def _process_tdl(self, out_name):
        if self._profile.has_option(self._section, 'tdl') is False:
            return

        tdls = self._profile.get(self._section, 'tdl').split(':')
        for tdl in tdls:
            shell_cmd = 'cp ' + out_name + ' ' + os.path.join(
                os.path.expandvars(tdl) + '/tdl/mod')
            shell_cmd += '; '
            shell_cmd += 'tdlupdate'
            shell_cmd += ' -m ' + self._remove_extension_name(out_name)
            shell_cmd += ' -r ' + os.path.join(
                os.path.expandvars(tdl) + '/tdl/mod')

            Log().get().info('[' + self._section + '] ' + shell_cmd)
            proc = subprocess.Popen([shell_cmd],
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE,
                                    shell=True)
            out, err = proc.communicate()

            # handle result
            if proc.returncode != 0:
                Log().get().error(err.decode('utf-8'))
                Log().get().error(out.decode('utf-8'))
                exit(proc.returncode)

        return

    def _process_dataset(self, out_name):
        out = ""
        err = ""

        if self._profile.has_option(self._section, 'dataset') is False:
            return

        datasets = self._profile.get(self._section, 'dataset').split(':')
        for dataset in datasets:
            shell_cmd = 'dlupdate ' + os.path.join(os.getcwd(),
                                                   out_name) + ' ' + dataset
            Log().get().info('[' + self._section + '] ' + shell_cmd)
            proc = subprocess.Popen([shell_cmd],
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE,
                                    shell=True)
            out, err = proc.communicate()

            # handle result
            if proc.returncode != 0:
                Log().get().error(err.decode('utf-8'))
                Log().get().error(out.decode('utf-8'))
                exit(proc.returncode)

        return

    def _process_file(self, in_name):
        out_name = in_name

        try:
            out_name = self._profile.get(self._section, 'file')
            out_name = out_name.replace("$OF_COMPILE_BASE",
                                        self._remove_extension_name(in_name))

            if in_name != out_name:
                Log().get().info('[' + self._section + '] ' + 'cp ' + in_name +
                                 ' ' + out_name)
                shutil.copy(in_name, out_name)

        except:
            Log().get().error('[' + self._section + '] failed to copy ' +
                              in_name)
            exit(-1)

        return out_name

    def run(self, in_name):
        # analyze section
        if self._analyze(in_name) < 0:
            return in_name

        # start section
        Log().get().debug("[" + self._section + "] start section")

        # update predefined environment variable
        base_name = self._remove_extension_name(in_name)
        out_name = base_name + '.' + self._remove_filter_name(self._section)
        Context().add_env('$OF_COMPILE_IN', in_name)
        Context().add_env('$OF_COMPILE_OUT', out_name)
        Context().add_env('$OF_COMPILE_BASE', base_name)

        # add environment variables
        for key in self._profile.options(self._section):
            value = self._profile.get(self._section, key)

            if key.startswith('$'):
                Context().add_env(key, value)

        # process others
        out_name = self._process_file(in_name)
        self._process_dataset(out_name)
        self._process_tdl(out_name)
        self._process_region(out_name)

        # set section as completed
        Context().set_section_complete(self._remove_filter_name(self._section))

        # end section
        Log().get().debug("[" + self._section + "] end section")

        return out_name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Set of functions useful in any module.

This module gathers a set of functions that are useful in many other modules. When a 
function is widely used in different modules, a general version of it is created and 
can be found here.

  Typical usage example:

  Context = Context()
"""

# Generic/Built-in modules
import os
from datetime import datetime

# Third-party modules

# Owned modules
from .Log import Log


class SingletonMeta(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(SingletonMeta,
                                        cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class Context(metaclass=SingletonMeta):
    _env = None
    _init_env = None
    _total_time = 0
    _init_pwd = None
    _mandatory = ""
    _cur_workdir = ""
    _root_workdir = ""
    _filter_dict = {}
    _const_tag = ""

    _section_complete_dict = {}

    def __init__(self):
        self._init_env = os.environ.copy()
        self._env = self._init_env
        self._init_pwd = os.getcwd()
        self.set_time_stamp()
        return

    def add_env(self, key, value):
        if key.startswith('$') is False:
            #warning
            return

        self._env[key[1:]] = os.path.expandvars(value)
        os.environ.update(self._env)

    def get_env(self):
        return self._env

    def set_section_complete(self, section):
        self._section_complete_dict[section] = True

    def is_section_complete(self, section):
        if section in self._section_complete_dict:
            return True
        return False

    def is_mandatory_complete(self):
        if self._mandatory in self._section_complete_dict:
            return True

        return False

    def set_time_stamp(self):
        self._time_stamp = datetime.now().strftime("_%Y%m%d_%H%M%S")

    def get_time_stamp(self):
        return self._time_stamp

    def set_root_workdir(self, workdir):
        self._root_workdir = workdir

    def get_root_workdir(self):
        return self._root_workdir

    def set_cur_workdir(self, cur_workdir):
        self._cur_workdir = cur_workdir

    def get_cur_workdir(self):
        return self._cur_workdir

    def set_mandatory_section(self, mandatory):
        index = mandatory.find('?')
        if index > 0:
            mandatory = mandatory[:index]

        self._mandatory = mandatory

    def get_mandatory_section(self):
        return self._mandatory

    def add_filter_result(self, key, value):
        self._filter_dict[key] = value

    def get_filter_result(self, key):
        result = False
        index = key.find('?')
        if index > 0:
            key = key[index:]

        if key in self._filter_dict:
            result = self._filter_dict[key]

        return result

    def set_const_tag(self, tag):
        """tag information should be erased"""
        if tag is not None:
            self._const_tag = "_" + tag

    def get_const_tag(self):
        return self._const_tag

    def clear(self):
        self._env = self._init_env
        os.environ.update(self._init_env)

        self._cur_workdir = ""
        self._mandatory = ""

        self.set_time_stamp()

        self._section_complete_dict.clear()
        self._filter_dict.clear()

        os.chdir(self._init_pwd)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

setup(
    setup_requires=[
        'pbr', 'setuptools', 'wheel', 'pytest', 'coverage', 'versioneer'
    ],
    pbr=True,
    use_scm_version=True,
)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""Description of the class in one sentence.

Description more in details.
"""
# Generic/Built-in modules
import subprocess
import shutil
import os

# Third-party modules

# Owned modules
from .Job import Job
from .Log import Log
from .Context import Context


class CompileJob(Job):

    def _analyze(self, in_name):
        # check if given section is already completed
        if Context().is_section_complete('setup') is False:
            Log().get().error(
                'cannot proceed due to setup not being completed.')
            exit(-1)

        # check if given section is already completed
        if Context().is_section_complete(self._section):
            Log().get().debug('section has already been processed. skipping [' +
                              self._section + '] section.')
            return -1

        # evaluate filter to decide whether this section should run or not
        if self._evaluate_filter(self._section, in_name) is False:
            Log().get().debug('[' + self._section + '] ' +
                              self._resolve_filter_name(self._section) +
                              ' is False. skipping section.')
            return -1

        return 0

    def _process_option(self):
        option = ""

        try:
            option = self._profile.get(self._section, 'option')
        except:
            Log().get().warning('option not specified in the ' + self._section +
                                ' section.')

        # build command
        shell_cmd = self._remove_filter_name(self._section) + " "
        shell_cmd += option
        shell_cmd = os.path.expandvars(shell_cmd)

        # run command
        #Log().get().info("shell_cmd: " + shell_cmd)
        Log().get().info('[' + self._section + '] ' + shell_cmd)
        env = Context().get_env()
        proc = subprocess.Popen([shell_cmd],
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE,
                                shell=True,
                                env=env)
        out, err = proc.communicate()

        # handle error
        if proc.returncode != 0:
            Log().get().error(err.decode('utf-8'))
            Log().get().error(out.decode('utf-8'))
            exit(proc.returncode)

        return

    def run(self, in_name):
        # analyze section
        if self._analyze(in_name) < 0:
            return in_name

        # start section
        Log().get().debug("[" + self._section + "] start section")

        # update predefined environment variable
        base_name = self._remove_extension_name(in_name)
        out_name = base_name + '.' + self._remove_filter_name(self._section)
        Context().add_env('$OF_COMPILE_IN', in_name)
        Context().add_env('$OF_COMPILE_OUT', out_name)
        Context().add_env('$OF_COMPILE_BASE', base_name)

        # add environment variables
        for key in self._profile.options(self._section):
            value = self._profile.get(self._section, key)

            if key.startswith('$'):
                Context().add_env(key, value)

            elif key.startswith('?'):
                self._add_filter(key, value, in_name)

            elif key == "option":
                self._process_option()
                out_name = Context().get_env().get('OF_COMPILE_OUT')
                if os.path.isfile(out_name) is not True:
                    out_name = in_name

        # set section as completed
        Context().set_section_complete(self._remove_filter_name(self._section))

        # end section
        Log().get().debug("[" + self._section + "] end section")

        return out_name
